"""
A miscellaneous utility for sequential labeling.
Copyright 2010,2011 Naoaki Okazaki.
"""

import optparse
import sys

def apply_templates(X, templates):
    """
    Generate features for an item sequence by applying feature templates.
    A feature template consists of a tuple of (name, offset) pairs,
    where name and offset specify a field name and offset from which
    the template extracts a feature value. Generated features are stored
    in the 'F' field of each item in the sequence.

    @type   X:      list of mapping objects
    @param  X:      The item sequence.
    @type   template:   tuple of (str, int)
    @param  template:   The feature template.
    """
    for template in templates:
        name = '|'.join(['%s[%d]' % (f, o) for f, o in template])
        for t in range(len(X)):
            values = []
            for field, offset in template:
                p = t + offset
                if p not in range(len(X)):
                    values = []
                    break
                values.append(X[p][field])
            if values:
                X[t]['F'].append('%s=%s' % (name, '|'.join(values)))

def readiter(fi, names, sep=' '):
    """
    Return an iterator for item sequences read from a file object.
    This function reads a sequence from a file object L{fi}, and
    yields the sequence as a list of mapping objects. Each line
    (item) from the file object is split by the separator character
    L{sep}. Separated values of the item are named by L{names},
    and stored in a mapping object. Every item has a field 'F' that
    is reserved for storing features.

    @type   fi:     file
    @param  fi:     The file object.
    @type   names:  tuple
    @param  names:  The list of field names.
    @type   sep:    str
    @param  sep:    The separator character.
    @rtype          list of mapping objects
    @return         An iterator for sequences.
    """
    X = []
    for line in fi:
        line = line.strip('\n')
        if not line:
            yield X
            X = []
        else:
            fields = line.split(sep)
            if len(fields) < len(names):
                raise ValueError(
                    'Too few fields (%d) for %r\n%s' % (len(fields), names, line))
            item = {'F': []}    # 'F' is reserved for features.
            for i in range(len(names)):
                item[names[i]] = fields[i]
            X.append(item)

def escape(src):
    """
    Escape colon characters from feature names.

    @type   src:    str
    @param  src:    A feature name
    @rtype          str
    @return         The feature name escaped.
    """
    return src.replace(':', '__COLON__')

def output_features(fo, X, field=''):
    """
    Output features (and reference labels) of a sequence in CRFSuite
    format. For each item in the sequence, this function writes a
    reference label (if L{field} is a non-empty string) and features.

    @type   fo:     file
    @param  fo:     The file object.
    @type   X:      list of mapping objects
    @param  X:      The sequence.
    @type   field:  str
    @param  field:  The field name of reference labels.
    """
    for t in range(len(X)):
        if field:
            fo.write('%s' % X[t][field])
        for a in X[t]['F']:
            if isinstance(a, str):
                fo.write('\t%s' % escape(a))
            else:
                fo.write('\t%s:%f' % (escape(a[0]), a[1]))
        fo.write('\n')
    fo.write('\n')

def to_crfsuite(X):
    """
    Convert an item sequence into an object compatible with crfsuite
    Python module.

    @type   X:      list of mapping objects
    @param  X:      The sequence.
    @rtype          crfsuite.ItemSequence
    @return        The same sequence in crfsuite.ItemSequence type.
    """
    import crfsuite
    xseq = crfsuite.ItemSequence()
    for x in X:
        item = crfsuite.Item()
        for f in x['F']:
            if isinstance(f, str):
                item.append(crfsuite.Attribute(escape(f)))
            else:
                item.append(crfsuite.Attribute(escape(f[0]), f[1]))
        xseq.append(item)
    return xseq

def main(feature_extractor, fields='w pos y', sep=' '):
    fi = sys.stdin
    fo = sys.stdout

    # Parse the command-line arguments.
    parser = optparse.OptionParser(usage="""usage: %prog [options]
This utility reads a data set from STDIN, and outputs attributes to STDOUT.
Each line of a data set must consist of field values separated by SEPARATOR
characters. The names and order of field values can be specified by -f option.
The separator character can be specified with -s option. Instead of outputting
attributes, this utility tags the input data when a model file is specified by
-t option (CRFsuite Python module must be installed)."""
        )
    parser.add_option(
        '-t', dest='model',
        help='tag the input using the model (requires "crfsuite" module)'
        )
    parser.add_option(
        '-f', dest='fields', default=fields,
        help='specify field names of input data [default: "%default"]'
        )
    parser.add_option(
        '-s', dest='separator', default=sep,
        help='specify the separator of columns of input data [default: "%default"]'
        )
    (options, args) = parser.parse_args()

    # The fields of input: ('w', 'pos', 'y) by default.
    F = options.fields.split(' ')

    if not options.model:
        # The generator function readiter() reads a sequence from a 
        for X in readiter(fi, F, options.separator):
            feature_extractor(X)
            output_features(fo, X, 'y')

    else:
        # Create a tagger with an existing model.
        import crfsuite
        tagger = crfsuite.Tagger()
        tagger.open(options.model)

        # For each sequence from STDIN.
        for X in readiter(fi, F, options.separator):
            # Obtain features.
            feature_extractor(X)
            xseq = to_crfsuite(X)
            yseq = tagger.tag(xseq)
            for t in range(len(X)):
                v = X[t]
                fo.write('\t'.join([v[f] for f in F]))
                fo.write('\t%s\n' % yseq[t])
            fo.write('\n')



__author__ = 'aleks'
#from sentence import *

#!/usr/bin/env python

"""
A feature extractor for OTE
"""

# Separator of field values.
separator = ' '

# Field names of the input data.ou
# CONLL_STRUCTURE = ['id', 'orth', 'lemma', 'pos', 'pos2', 'morph', 'parent', 'relation', '_1', '_2', 'as' ]
#CONLL_STRUCTURE = ['id', 'orth', 'lemma', 'pos', 'pos2', 'morph', 'parent', 'relation', '_1', '_2', 'y' ]
CONLL_STRUCTURE = ['id', 'orth', 'lemma', 'pos', 'pos2', 'morph', 'parent', 'relation', 'POSLemma' ,  'isTopLemma',  'topLemma',  'sent', 'isPointedToByRule', 'ruleId',   'y' ]

fields = " ".join( CONLL_STRUCTURE )


# Attribute templates.
templates = (

    (('ruleId', 0), ),

    (('isPointedToByRule', 0), ),

    (('relation', -2), ),
    (('relation', -1), ),
    (('relation',  0), ),
    (('relation',  1), ),
    (('relation',  2), ),
    (('relation', -2), ('relation', -1)),
    (('relation', -1), ('relation',  0)),
    (('relation',  0), ('relation',  1)),
    (('relation',  1), ('relation',  2)),

    (('pos', -2), ),
    (('pos', -1), ),
    (('pos',  0), ),
    (('pos',  1), ),
    (('pos',  2), ),
    (('pos', -2), ('pos', -1)),
    (('pos', -1), ('pos',  0)),
    (('pos',  0), ('pos',  1)),
    (('pos',  1), ('pos',  2)),
    (('pos', -2), ('pos', -1), ('pos',  0)),
    (('pos', -1), ('pos',  0), ('pos',  1)),
    (('pos',  0), ('pos',  1), ('pos',  2)),
    )


import crfutils

def feature_extractor(X):
    # Apply attribute templates to obtain features (in fact, attributes)
    crfutils.apply_templates(X, templates)
    if X:
        # Append BOS and EOS features manually
        X[0]['F'].append('__BOS__')     # BOS feature
        X[-1]['F'].append('__EOS__')    # EOS feature

if __name__ == '__main__':
    crfutils.main(feature_extractor, fields=fields, sep=separator)



# (1) prepare data by identifying sentiment words; output CONLL format, append column with an "S" where sentiment found
# python add_sentiment.py input_data/conll-format/input.conll input_data/conll-format/S_input.conll
import sys, os
import codecs

if __name__ == "__main__":

    if len(sys.argv) < 3:
        print "usage: add_sentiment.py <input_filename> <output_filename>"

    input_fname = sys.argv[1]
    output_fname = sys.argv[2]

    default_sent_fname = 'resources/slownikWydzwieku01.csv'

    if not os.path.isfile(input_fname):
        print "Input CONLL file is missing or is not readable"
        sys.exit(1)

    if not os.path.isfile(default_sent_fname):
        print "Default sentiment dictionary file is missing or is not readable"
        sys.exit(1)


    auto_senti = []
    with codecs.open(default_sent_fname,'r','utf8') as fr:
        for line in fr:
            toks = line.split('\t')
            if toks[2]=='1':
                auto_senti.append(toks[0])

    print "loaded",len(auto_senti),"words with sentiment"

    with codecs.open(input_fname,'r','utf8') as fr:
        with codecs.open(output_fname,'w','utf8') as fw:
            for line in fr:
                if len(line)>3:
                    toks = line.rstrip().split(' ')
                    if toks[2] in auto_senti:  # toks[2] = lemma
                        toks.append('S')
                    else:
                        toks.append('_')
                    fw.write(' '.join(toks))
                fw.write('\n')






import pickle, sys, os
import codecs
from SentencePatternMatcher import *

paths_fname = 'resources/pathsByIdWithMetadata.pkl'

if __name__ == "__main__":

    if len(sys.argv) < 3:
        print "usage: opta_patterns.py <input_filename> <output_filename>"

    input_fname = sys.argv[1]
    output_fname = sys.argv[2]

    if not os.path.isfile(input_fname):
        print "Input CONLL file is missing or is not readable"
        sys.exit(1)


    if not os.path.isfile(paths_fname):
        print "Dependency patterns file is missing or is not readable"
        sys.exit(1)

    spmatcher = SentencePatternMatcher()

    pathsByIdWithMetadata = pickle.load(open(paths_fname,'rb'))

    sentenceAsList = []
    with codecs.open(input_fname, 'r', 'utf8') as fr:
        with codecs.open(output_fname, 'w', 'utf8') as fw:

            for line in fr:
                if len(line) > 2:
                    sentenceAsList.append(line)
                else: # new sentence, parse what is loaded:

                    if len(sentenceAsList) > 0:
                        sentence = Sentence( sentenceAsList )
                        matchedTokensById = {}
                        startNodeIds = sentence.getTokenIDsWithAttribute('S')
                        for startNodeId in startNodeIds:
                            for id, p in pathsByIdWithMetadata.items():
                                tokenIDsFromRule = spmatcher.matchPathInSentence([startNodeId], p['path'] , sentence)
                                if tokenIDsFromRule is not None:
                                    for tokenId in tokenIDsFromRule:
                                        matchedTokensById[tokenId] = id

                        sentenceWithRuleId = []
                        for sline in sentenceAsList:
                            toks = sline.rstrip().split('\t')
                            tokenId = toks[0]
                            if tokenId in matchedTokensById:
                                toks.append( 'R' )
                                toks.append( matchedTokensById[tokenId] )
                            else:
                                toks.append( '_' )
                                toks.append( '_' )
                            toks.append('\n')
                            sentenceWithRuleId.append( ' '.join(toks))

                        for out_line in sentenceWithRuleId:
                            fw.write(out_line)
                        fw.write('\n')

                    sentenceAsList = []




